package outlab3;

import edu.princeton.cs.algs4.IndexMinPQ;
import edu.princeton.cs.algs4.StdOut;

/**
 * @author Ben Miller
 * @author Lauren Helbling
 * 
 *         With much help from algs4
 */
public class Djikstra {

    private double[] totalDist;
    private IndexMinPQ<Double> edgeQueue;
    private Edge[] edgeList;

    public Djikstra(WeightedGraph graph, int initial) {
        if (initial < 0 || initial > graph.getNumVertices()) {
            StdOut.println("Error, start vertex out of bounds");
            return;
        }

        totalDist = new double[graph.getNumVertices() + 1];
        edgeList = new Edge[graph.getNumVertices() + 1];

        for (int i = 0; i < totalDist.length; i++) {
            totalDist[i] = Double.POSITIVE_INFINITY;
        }
        totalDist[initial] = 0;

        edgeQueue = new IndexMinPQ<>(graph.getNumVertices() + 1);
        edgeQueue.insert(initial, totalDist[initial]);
        while (!edgeQueue.isEmpty()) {
            int vertex = edgeQueue.delMin();
            for (Edge edge : graph.getAdjacencylist()[vertex]) {
                relax(edge);
            }
        }

    }

    public void relax(Edge edge) {
        int start = edge.start;
        int finish = edge.finish;
        if (totalDist[finish] > totalDist[start] + edge.weight) {
            totalDist[finish] = totalDist[start] + edge.weight;
            edgeList[finish] = edge;
            if (edgeQueue.contains(finish))
                edgeQueue.decreaseKey(finish, totalDist[finish]);
            else
                edgeQueue.insert(finish, totalDist[finish]);
        }
    }

    public String pathTo(int destination) {
        StringBuilder result = new StringBuilder();
        if (totalDist[destination] == Double.POSITIVE_INFINITY)
            return "No Path";
        else if (totalDist[destination] == 0)
            return Integer.toString(destination);
        else {
            result.append(new StringBuilder(Integer.toString(destination)).reverse().toString());
            while (totalDist[destination] != 0) {
                result.append(" >- ");
                result.append(new StringBuilder(Integer.toString(edgeList[destination].start)).reverse().toString());
                destination = edgeList[destination].start;
            }
        }
        return result.reverse().toString();
    }

    public double getDistance(int destination) {
        return totalDist[destination];
    }
}
